/****************************************************************/
/* AMR VAF MEASUREMENT                                          */
/* 1999 copyright Nortel Networks                               */
/* All Rights reserved                                          */
/* This C code may only be used for AMR-NS selection phase for  */
/* the assesment of the impact on VAF                           */
/* Filename : main.h                                            */
/****************************************************************/
/* File History                                                 */
/* DATE         AUTHOR          DESCRIPTION                     */
/* 09/99        F.GABIN         Creation                        */
/****************************************************************/
#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/*---------------------------------------------------------------------------
---------------------------------------------------------------------------*
    The C code was succesfully compiled and ran on a
    PC/Windows environment. No make file is provided.

    VAF measurements
    performs the measurements of the ratio of voice frames over the amount
    of total encoded frames on the given input files

    What should be modified by the user of the source :

        in file main.h:
            Macros :
            initial dir of ARCON Files
            ARCON_BASE               "h:/hlaba"
            initial dir of COMSAT Files
            COMSAT_BASE              "h:/hlabc"
            Command line specific to the candidate
            for VAD option 1
            COMMAND_LINE_VAD1        "encoder  [-ns_on] -dtx"
            for VAD option 2
            COMMAND_LINE_VAD2        "encoder2 [-ns_on] -dtx"
            Report filename
            REPORT_FILENAME          "VAF_Report.txt"
            Candidate Acronym
            ARCON_CANDIDATE_ACRONYM  "xx"
            COMSAT_CANDIDATE_ACRONYM "xx"


   in file desc.c
   Group of files descriptions should be changed to match the file structure

 *--------------------------------------------------------------------------*/

#include "main.h"


/*---------------------------------------------------------------------------
 *                              Constants
 *--------------------------------------------------------------------------*/
#define FILENAME_SIZE                (29+(strlen(ARCON_BASE)>strlen(COMSAT_BASE) ? strlen(ARCON_BASE):strlen(COMSAT_BASE) ))

 /*---------------------------------------------------------------------------
 *                                Types
 *--------------------------------------------------------------------------*/
typedef struct {
   NoiseType  noise;
   int        total;
} StatNoise;


/*---------------------------------------------------------------------------
 *                                Functions
 *--------------------------------------------------------------------------*/
char *NextFile(FileDesc *);


/*---------------------------------------------------------------------------
 *                                Data
 *--------------------------------------------------------------------------*/
static char *gender[]={"f", "m"};


/*---------------------------------------------------------------------------
 *                                Main
 *--------------------------------------------------------------------------*/
int main(void)
{
   int n;
   Report* report;
   FILE  *Report_File;
   int  index_noise;
   /* Counters for statistics */
   /* Per noise types */
   int      nbFile[NB_NOISE_TYPES];
   int      nbFrames[NB_NOISE_TYPES];

   int      nbSpeech1[NB_NOISE_TYPES];
   int      nbDTXSeg1[NB_NOISE_TYPES];
   int      nbSid1[NB_NOISE_TYPES];

   int      nbSpeech2[NB_NOISE_TYPES];
   int      nbDTXSeg2[NB_NOISE_TYPES];
   int      nbSid2[NB_NOISE_TYPES];

   /* total */
   int      Total_File;
   int      Total_Frames;
   int      Total_Speech1;
   int      Total_DTXSeg1;
   int      Total_Sid1;
   int      Total_Speech2;
   int      Total_DTXSeg2;
   int      Total_Sid2;

   /* Means */
   float    VAF1_per_noise[NB_NOISE_TYPES];
   float    VAF1;
   float    VAF2_per_noise[NB_NOISE_TYPES];
   float    VAF2;

   for(index_noise=0;index_noise<NB_NOISE_TYPES;index_noise++) {
      nbFile[index_noise] = 0;
      nbFrames[index_noise] = 0;
      nbSpeech1[index_noise] = 0;
      nbDTXSeg1[index_noise] = 0;
      nbSid1[index_noise] = 0;
      nbSpeech2[index_noise] = 0;
      nbDTXSeg2[index_noise] = 0;
      nbSid2[index_noise] = 0;
   }



   /* Open report file */
   Report_File = fopen(REPORT_FILENAME,"wb");
   if(NULL == Report_File)
      {
      printf("Unable to open Report File %s \n", REPORT_FILENAME);
      exit(1);
      }

   /* Print definitions ...*/
   fprintf(Report_File, "Report for Voice Activity Factor measurements\n");
   fprintf(Report_File, "Definitions :\n");
   fprintf(Report_File, "Filename   : path and filename of the preprocessed file\n");
   fprintf(Report_File, "#frames    : total number of frames contained in the file\n");
   fprintf(Report_File, "#speech_1  : number of frames declared as SPEECH by the Tx SCR handler using VAD option 1\n");
   fprintf(Report_File, "#DTX_seg_1 : number of frames declared as SID_FIRST by the Tx SCR handler using VAD option 1 (i.e. number of silence segments)\n");
   fprintf(Report_File, "#Sid_1     : number of frames declared as SID_UPDATE by the Tx SCR handler using VAD option 1 \n");
   fprintf(Report_File, "#speech_2  : number of frames declared as SPEECH by the Tx SCR handler using VAD option 2\n");
   fprintf(Report_File, "#DTX_seg_2 : number of frames declared as SID_FIRST by the Tx SCR handler using VAD option 2 (i.e. number of silence segments)\n");
   fprintf(Report_File, "#Sid_2     : number of frames declared as SID_UPDATE by the Tx SCR handler using VAD option 2 \n");

   /* ... and column titles */
   fprintf(Report_File, "\n%-*s %10s %10s %10s %10s %10s %10s %10s\n",FILENAME_SIZE,"Filename","#frames","#speech_1","DTX_seg_1","#Sid_1","#speech_2","DTX_seg_2","#Sid_2");

   /* For every preprocessed files */
   for(n = 0; files[n].noise >= 0; n++)
      {
      char *fileName;
      /* Compute next filename */
      fileName = NextFile(&files[n]);
      index_noise = (int)(files[n].noise);

      while(fileName)
         {
         /* Compute VAF values for this file */
         report = Process(fileName, files[n].mode);
         /* store values for statistics */
         /* file counter */
         nbFile[index_noise]++ ;
         nbFrames[index_noise]+= report->nbFrame;
         /* VAD 1 */
         nbSpeech1[index_noise]+= report->nbSpeechFrame1;
         nbDTXSeg1[index_noise]+= report->nbDtxSegment1;
         nbSid1[index_noise]+= report->nbSidFrames1;
         /* VAD 2 */
         nbSpeech2[index_noise]+= report->nbSpeechFrame2;
         nbDTXSeg2[index_noise]+= report->nbDtxSegment2;
         nbSid2[index_noise]+= report->nbSidFrames2;

         /* Report values */
         fprintf(Report_File, "%*s ",FILENAME_SIZE,fileName);
         fprintf(Report_File, "%10d ",report->nbFrame);
         /* VAD 1 */
         fprintf(Report_File, "%10d ",report->nbSpeechFrame1);
         fprintf(Report_File, "%10d ",report->nbDtxSegment1);
         fprintf(Report_File, "%10d ",report->nbSidFrames1);
         /* VAD 2 */
         fprintf(Report_File, "%10d ",report->nbSpeechFrame2);
         fprintf(Report_File, "%10d ",report->nbDtxSegment2);
         fprintf(Report_File, "%10d \n",report->nbSidFrames2);

         fileName = NextFile(0);
         }
      }

   /* Compute statistics */
   /* Compute totals */
   Total_File    = 0;
   Total_Frames  = 0;
   /* VAD 1 */
   Total_Speech1 = 0;
   Total_DTXSeg1 = 0;
   Total_Sid1    = 0;
   /* VAD 2 */
   Total_Speech2 = 0;
   Total_DTXSeg2 = 0;
   Total_Sid2    = 0;

   for(index_noise=0;index_noise<NB_NOISE_TYPES;index_noise++)
      {
      Total_File    += nbFile[index_noise];
      Total_Frames  += nbFrames[index_noise];
      /* VAD 1 */
      Total_Speech1 += nbSpeech1[index_noise];
      Total_DTXSeg1 += nbDTXSeg1[index_noise];
      Total_Sid1    += nbSid1[index_noise];
      /* VAD 2 */
      Total_Speech2 += nbSpeech2[index_noise];
      Total_DTXSeg2 += nbDTXSeg2[index_noise];
      Total_Sid2    += nbSid2[index_noise];
      }

   /* compute means */
   /* Per noise types */
   for(index_noise=0;index_noise<NB_NOISE_TYPES;index_noise++)
      {

      if(nbFrames[index_noise] != 0)
         {
         /* VAD 1 */
         VAF1_per_noise[index_noise] = (float)nbSpeech1[index_noise]/nbFrames[index_noise];
         /* VAD 2 */
         VAF2_per_noise[index_noise] = (float)nbSpeech2[index_noise]/nbFrames[index_noise];
         }
      else
         {
         VAF1_per_noise[index_noise] = -1;
         VAF2_per_noise[index_noise] = -1;
         }
      }
   /* mean VAF */
   if(Total_Frames != 0)
      {
      VAF1 = (float)Total_Speech1/Total_Frames;
      VAF2 = (float)Total_Speech2/Total_Frames;
      }
   else
      {
      VAF1 = -1.0;
      VAF2 = -1.0;
      }

   /* Report statistics */
   fprintf(Report_File, "\nVoice Activity Factor Statistics\n");

   fprintf(Report_File, "%d files\n",Total_File);
   fprintf(Report_File, "%d frames\n",Total_Frames);

   /* total VAF */
   fprintf(Report_File, "VAF with VAD1 : %f \n",VAF1);
   fprintf(Report_File, "VAF with VAD2 : %f \n",VAF2);

   /* Per noise type */
   /* Car Noise */
   fprintf(Report_File, "VAF with VAD1 (car noise): %f \n",VAF1_per_noise[CAR_NOISE]);
   fprintf(Report_File, "VAF with VAD2 (car noise): %f \n",VAF2_per_noise[CAR_NOISE]);
   /* Street Noise */
   fprintf(Report_File, "VAF with VAD1 (street noise): %f \n",VAF1_per_noise[STREET_NOISE]);
   fprintf(Report_File, "VAF with VAD2 (street noise): %f \n",VAF2_per_noise[STREET_NOISE]);
   /* Babble Noise */
   fprintf(Report_File, "VAF with VAD1 (babble noise): %f \n",VAF1_per_noise[BABBLE_NOISE]);
   fprintf(Report_File, "VAF with VAD2 (babble noise): %f \n",VAF2_per_noise[BABBLE_NOISE]);
   /* Music */
   fprintf(Report_File, "VAF with VAD1 (music noise): %f \n",VAF1_per_noise[MUSIC]);
   fprintf(Report_File, "VAF with VAD2 (music noise): %f \n",VAF2_per_noise[MUSIC]);
   /* Talker */
   fprintf(Report_File, "VAF with VAD1 (talker noise): %f \n",VAF1_per_noise[TALKERS]);
   fprintf(Report_File, "VAF with VAD2 (talker noise): %f \n",VAF2_per_noise[TALKERS]);

   /* Close report file */
   fclose (Report_File);
   return(0);
}


/* ======================================================================== */

/*---------------------------------------------------------------------------
 *                                NextFile
 *--------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------
    Return next file in the list
    Either by returning the next filename within the group of preprocessed
    file descriptor or by moving to the next group of preprocessed
    file descriptor
 *--------------------------------------------------------------------------*/

char *NextFile(FileDesc *desc)
{
    static FileDesc *fileDesc = 0;
    static int currentCount = 0;
    static char name[500];
    char nametmp[500];
    int count;
    int g,id,s;

    if(desc != 0) {
        fileDesc = desc;
        currentCount = 0;
    }
    assert(fileDesc);

    count = 0;
    for(g = 0; g < 2; g++) { /* gender */
        for(id = 1; id <= 2; id++) { /* id */
            for(s = fileDesc->firstSample; s <= fileDesc->lastSample; s++) { /* sample */
                /* Seek filename number */
                if(count == currentCount) {
                    /* Construct the filename */
                    strcpy(name, fileDesc->base);
                    strcat(name, "/exp");
                    strcat(name, fileDesc->expNum);
                    strcat(name, "/lab");
                    strcat(name, fileDesc->testLabAcronym);
                    strcat(name, "/ns");
                    strcat(name, fileDesc->candidateAcronym);
                    strcat(name, "/");
                    strcat(name, gender[g]);
                    sprintf(nametmp, "%s%d", name, id);
                    strcat(nametmp, fileDesc->comsatException);
                    sprintf(name, "%s%*d", nametmp, fileDesc->sampleDigitNumber, s);
                    strcat(name, "c");
                    strcat(name, fileDesc->cond);
                    strcat(name, ".pre");
                    currentCount++;
                    return(name);
                    } /* if count */
                count++;
                } /* for s */
            } /* for id */
        } /* for g */
    return(0);
}


